/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.ui.editor;

import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createInterferenceLatencyTable;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import org.fortiss.af3.allocation.ui.editor.ParameterTableEditor;
import org.fortiss.af3.exploration.moea.model.allocation.InterferenceLatencyEntry;
import org.fortiss.af3.exploration.moea.model.allocation.InterferenceLatencyTable;
import org.fortiss.af3.exploration.moea.model.annotation.InterferenceLatency;
import org.fortiss.af3.platform.model.PlatformArchitecture;
import org.fortiss.af3.task.model.TaskArchitecture;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Editor for user-provided temporal properties of {@link TaskArchitecture}s (WCETs, periods).
 * 
 * @author diewald
 */
public class InterferenceLatencyTableEditor extends ParameterTableEditor<InterferenceLatencyTable> {

	/** Constructor. */
	public InterferenceLatencyTableEditor(PlatformArchitecture pArch) {
		super(InterferenceLatencyEntry.class, InterferenceLatency.class);

		if(pArch != null) {
			editedObject =
					pickFirstInstanceOf(InterferenceLatencyTable.class, pArch.getSpecifications());
			if(editedObject == null) {
				editedObject = createInterferenceLatencyTable();
				ITopLevelElement modelContext =
						IPersistencyService.getInstance().getTopLevelElementFor(pArch);
				modelContext.runAsCommand(() -> pArch.getSpecifications().add(editedObject));
			}
		}
	}
}
