/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode;

import org.aopalliance.intercept.MethodInterceptor;
import org.fortiss.af3.exploration.alg.module.DecoderModule;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Provides;
import com.google.inject.matcher.Matchers;

/**
 * {@link Module} that defines the method interceptors of {@link DecoderModule}s. This module must
 * be installed at the {@link Injector} that creates the concrete {@link DecoderModule}s. Otherwise,
 * the injection is not known at construction time of the {@link DecoderModule}s and thus method
 * interception is not operational.
 * 
 * @author diewald
 */
public class DecodingInterceptModule extends AbstractModule {

	/** Reference to the {@link DecodeInterceptor} handling the method interceptions. */
	private DecodeInterceptor decodingPluginInterceptor;

	/** Constructor. The given {@link DecodeInterceptor} is installed with the binder. */
	public DecodingInterceptModule(DecodeInterceptor decodingPluginInterceptor) {
		this.decodingPluginInterceptor = decodingPluginInterceptor;
	}

	/** Returns the {@link MethodInterceptor} handling Plugin-capable {@link DecoderModule}s. */
	@Provides
	public DecodeInterceptor get() {
		return decodingPluginInterceptor;
	}

	/** {@inheritDoc} */
	@Override
	public void configure() {
		bindInterceptor(Matchers.subclassesOf(DecoderModule.class),
				Matchers.annotatedWith(Decodes.class), decodingPluginInterceptor);
	}
}
