/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.evaluate.annotations;

import static com.google.common.base.Preconditions.checkNotNull;

import java.lang.annotation.Annotation;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.evaluate.ICompositeEvaluator;
import org.opt4j.core.problem.Evaluator;

/**
 * Implementation of the {@link Evaluate} annotation that carries a reference to the
 * {@link Evaluator} containing it.
 * 
 * @author diewald
 */
public class EvaluateImpl implements Evaluate {
	/**
	 * Evaluator class referenced by the evaluate method. Must be the containing {@link Evaluator}
	 * class.
	 */
	private final Class<? extends ICompositeEvaluator<?>> value;

	/** Constructor. */
	public EvaluateImpl(Class<? extends ICompositeEvaluator<?>> value) {
		this.value = checkNotNull(value, "value");
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Class<? extends ICompositeEvaluator> value() {
		return this.value;
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		// This is specified in java.lang.Annotation.
		return (127 * "value".hashCode()) ^ value.hashCode();
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(Object o) {
		if(!(o instanceof Evaluate)) {
			return false;
		}

		Evaluate other = (Evaluate)o;
		return value.equals(other.value());
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return "@" + Evaluate.class.getName() + "(value=" + value + ")";
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends Annotation> annotationType() {
		return Evaluate.class;
	}
}
