/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype;

import static org.fortiss.tooling.common.util.LambdaUtils.filterByType;

import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.module.DecoderModule;
import org.fortiss.af3.exploration.alg.service.module.IExplorationEncoding;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

/**
 * Base class for {@link Phenotype}s which consists of {@link Phenotype}s.
 * 
 * @author diewald
 */
public abstract class PhenotypeBase implements Phenotype, IExplorationEncoding {

	/** Flag to mark a phenotype as being processed by sub-Decoders. */
	private boolean isSubDecoding;

	/** Marks an encoding infeasible if a {@link DecoderModule} detects a constraint violation. */
	private boolean isInfeasible = false;

	/** Map holding the DSE-internal constraints. */
	protected List<InternalConstraint<?>> constraints;

	/** {@inheritDoc} */
	@Override
	public void setSubDecoding(boolean isSubDecoding) {
		this.isSubDecoding = isSubDecoding;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSubDecoding() {
		return isSubDecoding;
	}

	/** Returns isInvalid. */
	@Override
	public boolean isInfeasible() {
		return isInfeasible;
	}

	/** {@inheritDoc} */
	@Override
	public void setInfeasible() {
		isInfeasible = true;
	}

	/** {@inheritDoc} */
	@Override
	public <T extends InternalConstraint<?>> T getConstraint(Class<T> cType,
			Object constrainedElement) {
		for(T constraint : filterByType(constraints, cType)) {
			if(constrainedElement instanceof IRequestAdapter &&
					constraint.getSources().contains(constrainedElement)) {
				return constraint;
			} else if(constrainedElement instanceof IResourceAdapter &&
					constraint.getTargets().contains(constrainedElement)) {
				return constraint;
			}
		}

		return null;
	}
}
