/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch;

import java.util.Collection;

/**
 * Adapter between logical channels of the system meta-model used in the modeling
 * tool, and the DSE.
 * 
 * @author barner
 */
public interface ISignalAdapter extends IRequestAdapter {
	/** Temporal activation of the signal. */
	public enum SignalType {
		/** Purely event-based activation. */
		APERIODIC,
		/** Strictly periodic activation. */
		PERIODIC,
		/** Temporally upper-bounded, event-based activation. */
		SPORADIC
	}

	/** Returns the source component of this {@link ISignalAdapter}. */
	ITaskAdapter getSource();

	/** Returns the collection of target components of this {@link ISignalAdapter}. */
	Collection<ITaskAdapter> getTargets();

	/** Returns the target ports of the input models. */
	Collection<?> getTargetPorts();

	/** Returns the size of the signal sent via this {@link ISignalAdapter} in Bits. */
	long getSignalSize();

	/**
	 * Returns the {@link SignalType} of a message if it were constructed from this
	 * {@link ISignalAdapter}.
	 */
	SignalType getSignalType();

	/**
	 * Returns the transmission time of a message sent via this {@link ISignalAdapter} if it was
	 * deployed onto the given {@link ITransmissionUnitAdapter}.
	 */
	double getTransmissionTime(ITransmissionUnitAdapter transmissionUnit);
}
