/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidModelException;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.GatewayUnit;

/**
 * Adapter between the AF3 {@link GatewayUnit} and the DSE.
 * 
 * @author diewald
 */
public class AF3GatewayAdapter extends AF3ResourceAdapter<GatewayUnit>
		implements IGatewayUnitAdapter {

	/** Power consumption to transfer a message of n bits via this {@link GatewayUnit}. */
	private Double power;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @param gatewayUnit
	 *            {@link GatewayUnit} to create an adapter for.
	 * @throws InvalidModelException
	 *             if a required annotation cannot be found.
	 */
	public AF3GatewayAdapter(GatewayUnit gatewayUnit) throws InvalidModelException {
		super(gatewayUnit);
	}

	/** {@inheritDoc} */
	@Override
	public double getWCFT() {
		// TODO: add timing annotation for gateways
		// Currently, do nothing but returning 1.0s, since the timing annotation is missing for
		// Gateways
		return 0.000001;
	}

	/** {@inheritDoc} */
	@Override
	public double getPower(long messageSize) {
		// TODO: Add an appropriate bandwidth annotation for gateway units
		if(power == null) {
			return 1.0;
		}
		return power;
		// return messageSize * transmissionUnitBandwidth.getBandwidth_MBytesPerSecond() / 1e6;
	}

	/** {@inheritDoc} */
	@Override
	public double getTransmissionDuration(double messageSizeMB) {
		// TODO Auto-generated method stub
		return 0;
	}
}
