/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getFirstParentWithType;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemAccessTask;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidModelException;
import org.fortiss.af3.platform.model.MemoryUnit;
import org.fortiss.af3.platform.model.allocation.MemoryAllocationTable;
import org.fortiss.af3.task.model.Task;
import org.fortiss.af3.task.model.TaskArchitecture;
import org.fortiss.af3.task.model.allocation.TaskMemoryAllocationEntry;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Wraps AF3 {@link Task}s that perform purely memory accesses.
 * 
 * @author diewald
 */
public class AF3MemAccessTask extends AF3TaskAdapter implements IMemAccessTask {

	/** AER type of the task */
	private AER_MEM_TYPE taskType;

	/**
	 * References the {@link IMemoryUnitAdapter} holding the data of a task. This could be extended
	 * to a map in future to represent different data locations of a task's signals.
	 */
	private IMemoryUnitAdapter memLocation;

	/** Constructor. */
	public AF3MemAccessTask(Task af3Task, Collection<IComponentAdapter> components,
			Set<IExecutionUnitAdapter> execUnits, Set<IMemoryUnitAdapter> memUnits,
			AER_MEM_TYPE taskType) throws InvalidModelException {
		super(af3Task, components, execUnits);
		this.taskType = taskType;

		// Identifies which IMemoryUnit contains the given task
		TaskArchitecture ta = getFirstParentWithType(af3Task, TaskArchitecture.class);
		MemoryAllocationTable memAllocTbl = ta.getMemoryAllocationTable();
		List<IModelElement> af3MemUnits =
				memAllocTbl.getTargetElements(af3Task, TaskMemoryAllocationEntry.class);
		if(!af3MemUnits.isEmpty()) {
			Assert.isTrue(af3MemUnits.size() == 1);
			MemoryUnit af3MemUnit = (MemoryUnit)getFirst(af3MemUnits).get();
			memLocation =
					getFirst(memUnits, m -> m.getObject(MemoryUnit.class) == af3MemUnit).get();
		}
	}

	/** {@inheritDoc} */
	@Override
	public AER_MEM_TYPE getAERType() {
		return taskType;
	}

	/** Returns memLocation. */
	@Override
	public IMemoryUnitAdapter getMemoryLocation() {
		return memLocation;
	}
}
