/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.transformation;

import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;
import static org.fortiss.af3.partition.util.PartitionModelElementFactory.createPartition;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.feature.partition.variable.Partition;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;

import com.google.inject.Provides;

/**
 * Transforms a {@link PartitionMappingEncoding} to a {@link SuperSet} of AF3 Partitions.
 * 
 * @author diewald
 */
public class PartitionTransformation extends
		ExplorationTransformationModuleBase<SuperSet<org.fortiss.af3.partition.model.Partition>> {

	/** Associates internal partitions with generated AF3 partitions. */
	private Map<Partition, org.fortiss.af3.partition.model.Partition> internalExtAssoc =
			new HashMap<>();

	/** See {@link PartitionTransformation}. */
	@Transforms
	@Provides
	public SuperSet<org.fortiss.af3.partition.model.Partition>
			transform(@Genotyped PartitionMappingEncoding pmEncoding) {
		SuperSet<org.fortiss.af3.partition.model.Partition> partitions =
				createSuperSet("Partitions", org.fortiss.af3.partition.model.Partition.class);
		int i = 0;
		for(Partition internalPartition : pmEncoding.getRequesters()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					createPartition("Partition_" + i++);
			internalExtAssoc.put(internalPartition, af3Partition);
			partitions.getEntries().add(af3Partition);
		}
		return partitions;
	}

	/** Provides the associative map of internal partitions to generated AF3 partitions. */
	@Provides
	public Map<Partition, org.fortiss.af3.partition.model.Partition> getAssocMap() {
		return internalExtAssoc;
	}
}
