/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;

/**
 * Represents partitions executed by a hypervisor running on physical execution units.
 * The current implementation as an IExecutionUnitAdapter is a HACK to make the
 * PartitionMappingEncoding compatible with the TaskMappingEncoding. This should be refactored ASAP.
 * 
 * @author diewald
 */
public class Partition implements IExecutionUnitAdapter, IRequestAdapter {

	/** Caches whether an {@link Partition} has allocated entries (previous iteration). */
	private boolean empty = true;

	/** Returns whether an {@link Partition} has allocated entries (previous iteration). */
	public boolean isEmpty() {
		return empty;
	}

	/** Defines whether an {@link Partition} has allocated entries (this iteration). */
	public void setEmpty(boolean empty) {
		this.empty = empty;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isVirtual() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Double getFailureRate() {
		return 0.0;
	}

	/** {@inheritDoc} */
	@Override
	public double getInterferenceLatency(IExecutionUnitAdapter execUnit) {
		return 0;
	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		// No associated Object with a name at the moment.
		return toString();
	}

	/** {@inheritDoc} */
	@Override
	public <T> T getObject(Class<T> adpType) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public <T> void setObject(T element, Class<T> adpType) {
		// stub
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return "Partition_" + hashCode();
	}

	/** {@inheritDoc} */
	@Override
	public Partition clone() {
		try {
			return (Partition)super.clone();
		} catch(CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
	}
}
