/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable;

import static java.util.Collections.emptyList;

import java.util.Collection;
import java.util.Random;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.RandomExploration;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.Creates;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEncoding.CREATE_STARTEGY;
import org.fortiss.af3.exploration.alg.feature.taskmapping.ExecutionUnitSet;
import org.fortiss.af3.exploration.alg.module.CreatorModule;
import org.opt4j.core.problem.Creator;

import com.google.inject.Provides;

/**
 * {@link Creator} for {@link PartitionMappingEncoding}s.
 * 
 * @author diewald
 */
public class PartitionMappingCreator extends CreatorModule<PartitionMappingEncoding> {

	/** {@link Random} number generator for the DSE. */
	private RandomExploration rand = RandomExploration.getInstance();

	/**
	 * Creates an initial {@link PartitionMappingEncoding} using strategies using the minimal,
	 * maximal, or a random number of created {@link Partition}s w.r.t. the number of
	 * {@link ITaskAdapter}s
	 */
	@Provides
	@Creates
	@Genotyped
	public PartitionMappingEncoding create(ExecutionUnitSet execUnitSet,
			Collection<ITaskAdapter> tasks) {
		Set<IExecutionUnitAdapter> execUnits = execUnitSet.getExecutionUnits();

		int numTasks = tasks.size();
		// "Probability distribution" for the strategy selection: |1|numTasks - 2|1| ==> numTasks
		int strategySelIdx = rand.nextInt(numTasks);
		CREATE_STARTEGY strategy;
		if(strategySelIdx == 0) {
			strategy = CREATE_STARTEGY.MIN_SET;
		} else if(strategySelIdx == numTasks - 1) {
			strategy = CREATE_STARTEGY.MAX_SET;
		} else {
			strategy = CREATE_STARTEGY.RANDOM_SET;
		}

		return new PartitionMappingEncoding(strategy, numTasks, execUnits, emptyList());
	}

	/** {@inheritDoc} */
	@Override
	public PartitionMappingEncoding create() {
		// Required by the framework...
		return null;
	}
}
