/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.problem;

import java.util.HashSet;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Decodes;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.platform.variable.PlatformExecUnitInstanceEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.ExecutionUnitSet;
import org.fortiss.af3.exploration.alg.feature.taskmapping.PlatformExecUnitSetProvider;
import org.fortiss.af3.exploration.alg.module.DecoderModule;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;

/**
 * Creates a {@link ExecutionUnitSet} from the instances encoded in a
 * {@link PlatformExecUnitInstanceEncoding}. This set serves as mapping target for task or partition
 * mappings.
 * 
 * This decoder replaces the {@link PlatformExecUnitSetProvider} that also provides
 * {@link ExecutionUnitSet}s derived from an {@link PlatformCommunicationGraphEncoding}.
 * 
 * @author diewald
 */
public class PlatformExecUnitSetDecoder extends DecoderModule<ExecutionUnitSet> {

	/** See {@link PlatformExecUnitSetDecoder}. */
	@Provides
	@Decodes
	public ExecutionUnitSet decode(@Genotyped PlatformExecUnitInstanceEncoding peuiEnc) {
		Set<IExecutionUnitAdapter> instances = new HashSet<>();
		peuiEnc.getContainedInstanceTable().values().forEach(i -> {
			if(i instanceof IExecutionUnitAdapter) {
				instances.add((IExecutionUnitAdapter)i);
			}
		});
		return new ExecutionUnitSet(instances);
	}

	/** {@inheritDoc} */
	@Override
	public ExecutionUnitSet decode(Genotype arg0) {
		// Required by the framework.
		return null;
	}
}
