/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.safetyarch;

import static java.util.Arrays.asList;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISafetyFunctionAdapter;
import org.fortiss.af3.exploration.alg.feature.ExplorationFeature;
import org.fortiss.af3.exploration.alg.feature.IExplorationFeature;
import org.fortiss.af3.exploration.alg.feature.partition.variable.TaskPartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.safetyarch.problem.SFGraphDecoder;
import org.fortiss.af3.exploration.alg.feature.safetyarch.problem.SFMappingDecoder;
import org.fortiss.af3.exploration.alg.feature.safetyarch.problem.SafeTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.safetyarch.variable.SafetyFunctionArchCreator;
import org.fortiss.af3.exploration.alg.feature.safetyarch.variable.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.feature.safetyarch.variable.operator.MutateSafetyFunctionArchitectureModule;

import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.TypeLiteral;

/**
 * Explores which MooN architecture is suitable for the safety functions present in the input
 * models. This {@link IExplorationFeature} affects both the task model as well as the logical
 * architecture since the behaviour of the system could be modified.
 * 
 * @author diewald
 */
public class SafetyArchitectureExploration extends ExplorationFeature {

	/** Constructor. */
	@SuppressWarnings("unchecked")
	public SafetyArchitectureExploration() {
		super("Safety Architecture Exploration");

		// Variables
		addVariable(TypeLiteral.get(SafetyFunctionArchEncoding.class),
				new SafetyFunctionArchCreator(),
				asList(new MutateSafetyFunctionArchitectureModule()));
		// Decoding
		addProblem(TypeLiteral.get(TaskPartitionMappingEncoding.class), new SFMappingDecoder());
		// TODO: ConstraintDecoder
		// getService(IExplorationModuleService.class).registerProblem(
		// AbstractTaskMappingEncoding.class, new SFMappingConstraintDecoder());
		addProblem(TypeLiteral.get(SafeTaskGraphEncoding.class), new SFGraphDecoder());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		Set<ISafetyFunctionAdapter> sfAdps =
				dseInjector.getInstance(Key.get(new TypeLiteral<Set<ISafetyFunctionAdapter>>() {/**/
				}));
		return sfAdps.isEmpty() ? false : true;
	}
}
