/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling;

import static org.eclipse.core.runtime.Assert.isTrue;
import static org.fortiss.af3.exploration.alg.util.ExplorationAlgUtils.DSE_VALIDATE_SCHEDULE;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Decodes;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.scheduling.problem.SharedMemAccessLatencyEncoding;
import org.fortiss.af3.exploration.alg.feature.scheduling.problem.graph.ScheduleTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.scheduling.scheduler.EDFScheduler;
import org.fortiss.af3.exploration.alg.feature.scheduling.scheduler.EDFSchedulerBase;
import org.fortiss.af3.exploration.alg.feature.scheduling.scheduler.TDMABusScheduler;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.AcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.comm.MessageEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.DecoderModule;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;
import org.opt4j.core.Genotype;

import com.google.inject.Inject;
import com.google.inject.Provides;

/**
 * The Decoder decodes {@link Genotype}s into phenotype objects.
 * 
 * @author huang
 * @author diewald
 */
public class StrictTTDecoder<T extends IExecutionUnitAdapter, E extends TaskMappingEntry<ITaskAdapter, T>, M extends IMappingEncoding<ITaskAdapter, T, E, ?>>
		extends DecoderModule<StrictTTSchedule<T, E, M>> {

	/** Underlying bus scheduler */
	private TDMABusScheduler simpleBusScheduler;

	/**
	 * Constructs a new decoder that constructs {@link StrictTTSchedule}s for a given exploration
	 * run.
	 */
	@SuppressWarnings("unchecked")
	@Inject
	public StrictTTDecoder() {
		simpleBusScheduler = new TDMABusScheduler();
	}

	/** See {@link StrictTTDecoder}. */
	// FIXME: Dirty casts: Remove the generics?
	@Provides
	@Decodes
	public StrictTTSchedule decode(
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> itmEnc,
			MessageEncoding msgEnc, ScheduleTaskGraphEncoding stgEnc,
			AcyclicTaskGraphEncoding acycItgEnc, PlatformCommunicationGraphEncoding pcgEncoding,
			SharedMemAccessLatencyEncoding shMemEnc) {
		// TODO: Externalize the validation such that the encoding is not required to be passed to
		// the constructor (is there for validation purposes)

		EDFSchedulerBase<T, E, M> simpleEDFScheduler =
				(EDFSchedulerBase<T, E, M>)new EDFScheduler();
		StrictTTSchedule<T, E, M> schedule =
				simpleEDFScheduler.schedule(stgEnc, (M)itmEnc, shMemEnc);
		if(DSE_VALIDATE_SCHEDULE) {
			isTrue(schedule.validateSchedule(true));
		}

		simpleBusScheduler.schedule(schedule, msgEnc.getPeriodicMessages(), pcgEncoding);
		if(DSE_VALIDATE_SCHEDULE) {
			isTrue(schedule.validateSchedule(true));
		}

		return schedule;
	}

	/** {@inheritDoc} */
	@Override
	public StrictTTSchedule<T, E, M> decode(Genotype genotype) {
		return null;
	}
}
