/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling.evaluator;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.evaluate.annotations.Evaluate;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.scheduling.ResourceSchedule;
import org.fortiss.af3.exploration.alg.feature.scheduling.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.feature.scheduling.TimeSlot;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.EvaluatorModule;
import org.fortiss.af3.exploration.moea.model.objectives.LatencyMinObjective;
import org.opt4j.core.DoubleValue;

import com.google.inject.Provides;

/**
 * Creates a sum of the ending times of the last scheduled {@link TimeSlot}s allocated to
 * {@link IExecutionUnitAdapter}s.
 * 
 * @author diewald
 */
public class MinLatencyObjectiveEvaluator<T extends IExecutionUnitAdapter, E extends TaskMappingEntry<ITaskAdapter, T>, M extends TaskMappingEncodingBase<ITaskAdapter, T, E, ?>>
		extends EvaluatorModule<Double, LatencyMinObjective, StrictTTSchedule> {

	/** Creates a template energy objective evaluator */
	public MinLatencyObjectiveEvaluator(Class<LatencyMinObjective> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Creates an energy objective evaluator */
	public MinLatencyObjectiveEvaluator(LatencyMinObjective explorationTarget) {
		super(explorationTarget);
	}

	/** See {@link MinLatencyObjectiveEvaluator}. */
	@SuppressWarnings("unchecked")
	@Provides
	@Evaluate(MinLatencyObjectiveEvaluator.class)
	public DoubleValue evaluate(@SuppressWarnings("rawtypes") StrictTTSchedule schedule) {
		if(schedule.getResources().isEmpty()) {
			return new DoubleValue(Double.MAX_VALUE);
		}

		double latencySum = 0;

		// TODO(AD): Avoid raw casting by specifying the generics.
		for(IResourceAdapter resource : (Collection<IResourceAdapter>)(Collection<?>)schedule
				.getResources()) {
			if(!(resource instanceof IExecutionUnitAdapter)) {
				continue;
			}
			ResourceSchedule localSchedule = schedule.getResourceSchedule(resource);
			TimeSlot lastScheduledSlot = localSchedule.lastKey();
			latencySum += lastScheduledSlot.getEndTime();
		}

		return new DoubleValue(latencySum);
	}
}
