/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling.problem.graph;

import static java.util.Collections.unmodifiableCollection;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.PhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.AcyclicTaskGraph;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.AcyclicTaskGraphEncoding;

/**
 * Collection of acyclic task graphs that are used as an input for schedulers that encode data
 * dependencies and unroll tasks to task instance in case of multi-period scheduling.
 * 
 * @author diewald
 */
public class ScheduleTaskGraphEncoding extends PhenotypeBase {

	/** Maps {@link IScheduleTask}s to the {@link ScheduleTaskGraph}s containing them. */
	private Map<IScheduleTask, ScheduleTaskGraph> taskScheduleGraphMap = new HashMap<>();

	/** Constructor. */
	public ScheduleTaskGraphEncoding(AcyclicTaskGraphEncoding atgEnc) {
		for(AcyclicTaskGraph atg : atgEnc.getTaskGraphs()) {
			ScheduleTaskGraph stg = new ScheduleTaskGraph(atg);
			stg.getTasks().forEach(t -> taskScheduleGraphMap.put(t, stg));
		}
	}

	/** Copy Constructor. */
	public ScheduleTaskGraphEncoding(ScheduleTaskGraphEncoding encoding) {
		// WARNING: causes threading interferences. See the IMappingEncoding for a solution.
		taskScheduleGraphMap.putAll(encoding.taskScheduleGraphMap);
	}

	/** Returns all {@link ScheduleTaskGraph}s of this encoding. */
	public Collection<ScheduleTaskGraph> getGraphs() {
		return unmodifiableCollection(taskScheduleGraphMap.values());
	}

	/** Returns the ScheduleTaskGraph hosting the given {@link IScheduleTask}. */
	public ScheduleTaskGraph getGraphOf(IScheduleTask task) {
		return taskScheduleGraphMap.get(task);
	}
}
