/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling.scheduler;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemAccessTask;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.scheduling.problem.SharedMemAccessLatencyEncoding;
import org.fortiss.af3.exploration.alg.feature.scheduling.problem.graph.IScheduleTask;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;

/**
 * Simple Multicore EDF task scheduler.
 * 
 * @author huang
 * @author barner
 * @author diewald
 */
public class EDFScheduler extends
		EDFSchedulerBase<IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>> {

	/**
	 * Returns the execution time for the currently scheduled task. If the task is a memory
	 * read/write operation block, the access /
	 */
	@Override
	protected Double getDuration(
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> encoding,
			SharedMemAccessLatencyEncoding shMemEnc, IScheduleTask schedTask,
			TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter> entry) {
		Double executiontime = schedTask.getWcet(entry.getTarget());
		if(schedTask instanceof IMemAccessTask) {
			executiontime = shMemEnc.getLatencyOf(schedTask).doubleValue();
		}
		return executiontime;
	}
}
