/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping.problem.comm;

import static org.fortiss.tooling.common.util.LambdaUtils.filterSet;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.PhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISignalAdapter;
import org.fortiss.af3.exploration.alg.service.module.IExplorationEncoding;

/**
 * {@link IExplorationEncoding} that defines the {@link Message}s which are transmitted on a target
 * platform subject to some mapping.
 * 
 * @author diewald
 */
public class MessageEncoding extends PhenotypeBase implements IExplorationEncoding {

	/** Contains the set of generated messages. */
	Collection<Message> messages;

	/** Constructor. */
	public MessageEncoding(Collection<Message> messages) {
		this.messages = messages;
	}

	/** Returns the collection of all generated messages. */
	public Collection<Message> getAllMessages() {
		return messages;
	}

	/** Returns all messages that are sent periodically. */
	public Collection<Message> getPeriodicMessages() {
		return filterSet(messages,
				m -> m.getSignalAdapter().getSignalType() == ISignalAdapter.SignalType.PERIODIC);
	}
}
