/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping.variable;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.Creates;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.ExecutionUnitSet;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.TaskGraphEncoding;

import com.google.inject.Provides;
import com.google.inject.TypeLiteral;

/**
 * Creates a {@link TaskExUMappingEncoding}. The actual initialization code is found in the
 * encoding for reuse.
 * 
 * @author diewald
 */
public class TaskExUMappingCreator extends
		TaskMappingCreatorBase<IExecutionUnitAdapter, TaskExUMappingEntry, TaskExUMappingEncoding> {

	/** See {@link TaskExUMappingCreator}. */
	@Creates
	@Provides
	@Genotyped
	public TaskExUMappingEncoding create(ExecutionUnitSet execUnits, TaskGraphEncoding tgEnc) {
		return new TaskExUMappingEncoding(execUnits, tgEnc, constraints.get(), true);
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getSourceType() {
		return TypeLiteral.get(ITaskAdapter.class);
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getTargetType() {
		return TypeLiteral.get(IExecutionUnitAdapter.class);
	}

	/** {@inheritDoc} */
	@Override
	public TaskExUMappingEncoding create() {
		// Not needed, imposed by the framework
		return null;
	}
}
