/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.encoding;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.guava.IAssociativeElement;
import org.fortiss.af3.exploration.alg.module.common.operator.copy.Copyable;

/**
 * External interface that provides the entries of an {@link IMappingEncoding}.
 * 
 * @author diewald
 */
public interface IMappingEntry<S extends IRequestAdapter, T extends IResourceAdapter>
		extends IAssociativeElement<IMappingEntry<S, T>>, Copyable<IMappingEntry<S, T>> {

	/** Returns the {@link IRequestAdapter} that is represented by this {@link IMappingEntry}. */
	S getSource();

	/** Return target resource to which the request is mapped. */
	T getTarget();

	/**
	 * Reassigns the mapping entry to the given {@link IResourceAdapter} to account for changed
	 * underlying resource sets..
	 */
	void resetTarget(T resource);
}
