/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.evaluator.mapping;

import static org.junit.Assert.assertNotNull;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.TaskMapper;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;
import org.fortiss.af3.exploration.dseml.model.arithmetic.IArithmeticExpression;
import org.fortiss.af3.exploration.dseml.model.expression.IExpression;
import org.fortiss.af3.exploration.dseml.model.function.Maximize;
import org.fortiss.af3.exploration.dseml.model.function.Minimize;
import org.fortiss.af3.exploration.model.ExplorationObjective;

/**
 * Parses an {@link IExpression} of {@link ExplorationObjective}s.
 * 
 * @author diewald
 */
public class MappingParserObjective extends TaskMappingParser<Double> {

	/** Constructor. */
	public MappingParserObjective(
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> mapping,
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> virtMapping,
			IExpression expression, TaskMapper taskMapper) {
		super(mapping, virtMapping, expression, taskMapper);
	}

	/** {@inheritDoc} */
	@Override
	public Double parse() throws Exception {
		// For ExplorationObjectives the expression begins either with a Minimize or Maximize
		// operation.
		// FIXME: bad dsl design...
		IArithmeticExpression minMaxTerm = null;
		if(expression instanceof Minimize) {
			minMaxTerm = ((Minimize)expression).getArgs();
		} else if(expression instanceof Maximize) {
			minMaxTerm = ((Maximize)expression).getArgs();
		}
		assertNotNull(minMaxTerm);

		// The minMax Term is always non-zero.
		// FIXME: Using "Number" as a type is a hack: It is a workaround to comply with the
		// construction of objectives in the UI.
		return minMaxTerm.accept(this, Number.class).doubleValue();
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Maximize max) throws Exception {
		// Max/Min operators are handled in the EvaluationWrapper (pre-processing).
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Minimize min) throws Exception {
		// Max/Min operators are handled in the EvaluationWrapper (pre-processing).
		return null;
	}
}
