/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import java.util.HashSet;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Provides all known {@link IResourceAdapter}s of the target platform.
 * 
 * @author diewald
 */
public class AllResourceInputModule extends InputModule<Set<IGatewayUnitAdapter>> {

	/** See {@link TransmissionUnitInputModule}. */
	@InputProvider
	@Provides
	@Singleton
	public Set<IResourceAdapter> createTransmissionUnits(Set<IExecutionUnitAdapter> execUnits,
			Set<IGatewayUnitAdapter> gateways, Set<ITransmissionUnitAdapter> transUnits,
			Set<IMemoryUnitAdapter> memUnits) {
		Set<IResourceAdapter> allResources = new HashSet<>();
		allResources.addAll(execUnits);
		allResources.addAll(gateways);
		allResources.addAll(transUnits);
		allResources.addAll(memUnits);
		return allResources;
	}
}
