/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import java.util.HashSet;
import java.util.Set;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3.AF3ComponentAdapter;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Input module to read components that specify behavior and the data flow.
 * 
 * @author diewald
 */
public class ComponentInputModule extends InputModule<Set<IComponentAdapter>> {

	/** Provides the {@link ITaskAdapter}s of the task architecture. */
	@Provides
	@InputProvider
	@Singleton
	public Set<IComponentAdapter> getComponents(SuperSet<Component> af3Comps) {
		Set<IComponentAdapter> components = new HashSet<>();
		for(Component currentComponent : af3Comps.getEntries()) {
			components.add(new AF3ComponentAdapter(currentComponent));
		}
		return components;
	}
}
