/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import java.util.HashSet;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3.AF3ExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.platform.model.ExecutionUnit;

import com.google.inject.Module;
import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * {@link Module} providing the set of extracted {@link IExecutionUnitAdapter}s of the target
 * platform.
 * 
 * @author diewald
 */
public class ExecutionUnitTargetModule extends InputModule<Set<IExecutionUnitAdapter>> {

	/** Provides the {@link IExecutionUnitAdapter}s of the target platform. */
	@InputProvider
	@Provides
	@Singleton
	public Set<IExecutionUnitAdapter> getExUTargets(SuperSet<ExecutionUnit> af3ExecUnits) {
		Set<IExecutionUnitAdapter> execUnitTargets = new HashSet<>();
		af3ExecUnits.getEntries().forEach(e -> execUnitTargets
				.add(new AF3ExecutionUnitAdapter(e, af3ExecUnits.getEntries())));
		return execUnitTargets;
	}
}
