/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.operator.mutate.mapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;

/**
 * Base class for mutate operator that manipulate the redundancy of allocations.
 * 
 * @author huang
 */
public abstract class MutateRedundancy<S extends TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, T extends TaskMappingEncodingBase<ITaskAdapter, IExecutionUnitAdapter, S, ?>>
		extends MutateTasksBase<S, T> {
	/** Constructs a new redundancy mutation operator for a given exploration run */
	MutateRedundancy(Collection<ITaskAdapter> tasks) {
		super(tasks);
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(T encoding, double rate) {
		for(ITaskAdapter deployableComponent : encoding.getRequesters()) {
			if(rand.nextDouble() < rate) {
				// TODO(#3264)
				// if(systemModelAdapter.getReplicableComponents().contains(deployableComponent) {
				// TODO: Model minimum and maximum replication count as constraint?
				// Do the actual mutation of the allocation of the current request
				try {
					int minReplication = deployableComponent.getMinReplication();
					int maxReplication = deployableComponent.getMaxReplication();
					// TODO: Get the replication constraint here.
					// Collection<ReplicationConstraint> replConstraints =
					// encoding.getConstraintsOf(ReplicationConstraint.class);
					// Optional<ReplicationConstraint> curReplConstraint = replConstraints.stream()
					// .filter(c -> c.getRequester() == deployableComponent).findAny();
					// if(curReplConstraint.isPresent()) {
					// minReplication = curReplConstraint.get().getMin();
					// maxReplication = curReplConstraint.get().getMax();
					// }

					mutate(encoding, deployableComponent, minReplication, maxReplication);
				} catch(ExplorationException e) {
					// Opt4J does not allow to throw Exceptions here.
					throw new RuntimeException(e);
				}
				// }
				// TODO: What about temporal redundancy at the fixed resource
			}
		}
	}

	/**
	 * Performs the mutation of a single allocation
	 * 
	 * @throws ExplorationException
	 *             if no valid allocation targets can be identified for one {@link ITaskAdapter}.
	 */
	protected abstract void mutate(T encoding, ITaskAdapter deployableComponent,
			int minReplicatation, int maxReplication) throws ExplorationException;
}
