/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.expspectransf.constraint;

import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createCouplingConstraint;
import static org.fortiss.af3.exploration.alg.util.DseMLUtils.getCouplingRequesterSet;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.TaskMapper;
import org.fortiss.af3.exploration.alg.service.module.base.ExpSpecTransformatorBase;
import org.fortiss.af3.exploration.dseml.model.booleanp.allocation.ILocationExpression;
import org.fortiss.af3.exploration.dseml.model.expression.Set;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.CouplingConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.task.model.Task;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.TypeLiteral;

/**
 * Converts an allocation pattern {@link ExplorationConstraint} to an {@link InternalConstraint}.
 * 
 * @author diewald
 */
public final class CouplingPatternConverter
		extends ExpSpecTransformatorBase<ExplorationConstraint<?>, InternalConstraint<?>> {

	/** {@inheritDoc} */
	@Override
	public boolean outputIsReplacement() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public InternalConstraint<?> transform(Injector mainInjector,
			ExplorationConstraint<?> constraint) {
		TaskMapper taskMapper = mainInjector.getInstance(Key.get(new TypeLiteral<TaskMapper>() { // ...
		}));

		ILocationExpression locExpr = getCouplingRequesterSet(constraint);

		if(locExpr == null) {
			return null;
		}

		Set<IModelElement> requesterSet = locExpr.getLeft().getSetReference();
		Set<IModelElement> resourceSet = locExpr.getRight().getSetReference();

		TypeLiteral<?> srcType = TypeLiteral.get(mapAF3Type(requesterSet.getEntryType()));
		TypeLiteral<?> tgtType = TypeLiteral.get(mapAF3Type(resourceSet.getEntryType()));

		Collection<IRequestAdapter> reqSet = new ArrayList<>();
		requesterSet.getEntries().forEach(me -> reqSet.add(taskMapper.getAdapter((Task)me)));

		CouplingConstraint couplingConstr = createCouplingConstraint(reqSet, srcType, tgtType);
		return couplingConstr;
	}

	// TODO(AD): Remove or generalize. Obviously, this method should not exist at all...
	/** Translates AF3 types to its internal representation type. */
	private Class<?> mapAF3Type(Class<?> af3Type) {
		if(Task.class.isAssignableFrom(af3Type)) {
			return ITaskAdapter.class;
		} else if(ExecutionUnit.class.isAssignableFrom(af3Type)) {
			return IExecutionUnitAdapter.class;
		}
		return null;
	}
}
