/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.EncodingProviderModule;
import org.fortiss.af3.exploration.alg.feature.IExplorationFeature;
import org.fortiss.af3.exploration.alg.module.InputModule;

/**
 * {@link IExplorationService} that manages {@link InputModule}s, which provide system model
 * elements to other D3SE modules.
 * 
 * @author diewald
 */
public interface IExplorationInputService extends IExplorationService {

	/**
	 * Registers the given {@link InputModule} with this service.
	 * 
	 * @param inputModule
	 *            {@link InputModule} to register with the DSE.
	 */
	void registerExplorationInput(InputModule<?> inputModule);

	/**
	 * Returns the registered {@link InputModule}s, except those that provide artifacts of the given
	 * {@code excludedInputs}.
	 * 
	 * @param enabledFeatures
	 *            collection of features enabled for a DSE run.
	 * @param systemModelProviders
	 *            Provider of inputs used in the {@link InputModule}s for their construction.
	 * @return {@link EncodingProviderModule} that is able to provide all inputs used internally.
	 */
	EncodingProviderModule<?> getInputProviderModule(
			Collection<IExplorationFeature> enabledFeatures,
			EncodingProviderModule<?> systemModelProviders);
}
