/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.EncodingProviderModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.ProvidingPhenotype;
import org.fortiss.af3.exploration.alg.feature.IExplorationFeature;
import org.fortiss.af3.exploration.alg.service.module.IExplorationTransformationModule;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.exploration.model.SuperSetMap;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.inject.TypeLiteral;

/**
 * The {@link IExplorationTransformationService} transforms a given {@link ProvidingPhenotype} into
 * a set of {@link SuperSet}s describing the solution using types from the scope of the input
 * models. These types are the elements of the {@link SuperSet}s.
 * <p>
 * Each {@link SuperSet} defined as an output is produced by an independent
 * {@link IExplorationTransformationModule}. Such a module must specify a method annotated with a
 * {@link Transforms} annotation whose input parameters are provided by the Guice dependency
 * injection framework.
 * 
 * @author diewald
 */
public interface IExplorationTransformationService extends IExplorationService {

	/**
	 * Transforms elements of the given {@link ProvidingPhenotype} into {@link SuperSet}s encoded in
	 * the output {@link SuperSetMap}. The produced {@link SuperSet}s are defined by the passed list
	 * of {@link SuperSet} types.
	 * 
	 * @param enabledFeatures
	 *            collection of features that were enabled for a particular DSE run.
	 * @param inputProvider
	 *            Provider of the input sets to construct the exploration problem.
	 * @param phenotype
	 *            Source DSE encodings.
	 * @param outputSuperSets
	 *            Collection of {@link SuperSet} types to produces
	 * @return {@link SuperSetMap} of the output {@link SuperSet}s.
	 */
	<T extends IModelElement> SuperSetMap transform(Collection<IExplorationFeature> enabledFeatures,
			EncodingProviderModule<?> inputProvider, ProvidingPhenotype phenotype,
			Collection<TypeLiteral<SuperSet<? extends IModelElement>>> outputSuperSets);

	/** Registers the given {@link IExplorationTransformationModule} with the service. */
	void registerTransformator(IExplorationTransformationModule<?> transformator);
}
